@@ -3,6 +3,8 @@ from snowday.objects.account import Role
 from snowday.acl.grants.base import BaseGrant
 from snowday.acl.grants.util import get_privilege
 
+from snowday.types.base import Sql
+
 
 @dataclass
 class AccountGrant(BaseGrant):
@@ -11,12 +13,16 @@ class AccountGrant(BaseGrant):
         return get_privilege(self.__class__.__name__)
 
     @property
-    def grant(self):
-        return f"grant {self.privilege} on account to role {self.grantee.name};"
+    def assign(self):
+        return Sql(
+            statement=f"grant {self.privilege} on account to role {self.grantee.name};"
+        )
 
     @property
     def revoke(self):
-        return f"revoke {self.privilege} on account from role {self.grantee.name};"
+        return Sql(
+            statement=f"revoke {self.privilege} on account from role {self.grantee.name};"
+        )
 
 
 @dataclass
