@@ -15,6 +15,9 @@ def get_privilege(class_name: str):
     """A helper function for getting the name of a
     class's privilege, based on the assumption that
     the privilege will be included in the name.
+    ie:
+        AllGrant -> "all"
+        SelectGrant -> "select"
     """
     return get_priv_from_cls(class_name, 0, -1)
 
@@ -24,5 +27,8 @@ def get_future_privilege(class_name: str):
     class's future privilege, based on the assumption
     that the privilege will be included in the name
     and sandwiched between "future" and "grant".
+    ie:
+        FutureApplyGrant -> "apply"
+        FutureAllGrant -> "all"
     """
     return get_priv_from_cls(class_name, 1, -1)
