@@ -1,5 +1,6 @@
+from datetime import datetime
 from dataclasses import dataclass, field
-from typing import List
+from typing import List, Union
 
 from snowday.objects.base import BaseSnowflakeObject
 from snowday.exceptions import (
@@ -7,9 +8,61 @@ from snowday.exceptions import (
     InvalidWhitespaceException,
     UnsupportedFeatureException,
 )
+from snowday.types.base import Sql
 
 
-# https://docs.snowflake.com/en/sql-reference/sql/create.html
+EDITIONS = ["standard", "enterprise", "business_critical"]
+
+REGIONS = [
+    # aws
+    "us-east-1",
+    "us-east-2",
+    "us-west-2",
+    "ca-central-1",
+    "eu-central-1",
+    "eu-west-1",
+    "ap-northeast-1",
+    "ap-south-1",
+    "ap-southeast-1",
+    "ap-southeast-2",
+    # azure
+    "east-us-2",
+    "west-us-2",
+    "canada-central",
+    "us-gov-virginia",
+    "west-europe",
+    "switzerland-north",
+    "southeast-asia",
+    "australia-east",
+    # gcp
+    "us-central1",
+    "europe-west2",
+    "europe-west4",
+]
+
+WAREHOUSE_SIZES = [
+    "xsmall",
+    "small",
+    "medium",
+    "large",
+    "xlarge",
+    "xxlarge",
+    "xxxlarge",
+]
+
+SCALING_POLICIES = ["economy", "standard"]
+
+RESOURCE_MONITOR_TRIGGER_ACTIONS = ["suspend", "suspend_immediate", "notify"]
+
+RESOURCE_MONITOR_FREQUENCIES = [
+    "monthly",
+    "daily",
+    "weekly",
+    "yearly",
+    "never",
+]
+
+EXTERNAL_OAUTH_TYPES = ["okta", "azure", "ping_federate", "custom"]
 
 
 @dataclass
@@ -19,40 +72,48 @@ class Account:
     admin_name: str
     admin_password: str
     email: str
-    edition: str  # either STANDARD or ENTERPRISE or BUSINESS_CRITICAL
+    edition: str
     region: str
     comment: str
 
-    def __post_init__(self):
-        # Make sure edition is valid
-        if self.edition not in ["STANDARD", "ENTERPRISE", "BUSINESS_CRITICAL"]:
+    def _validate(self):
+        # FIXME! Make me run on init
+        if self.edition.lower() not in EDITIONS:
             raise InvalidParamsException(
-                "Edition must be STANDARD, ENTERPRISE, or BUSINESS_CRITICAL"
+                f"{self.edition} is not a valid snowflake edition. Options include: {EDITIONS}"
+            )
+        if self.region.lower() not in REGIONS:
+            raise InvalidParamsException(
+                f"{self.region} is not a valid snowflake region. Options include: {REGIONS}"
             )
-        # Make sure region is valid. FIXME!
-        # Ensure names are uppercased.
         self.name = self.name.upper()
 
     @property
     def create(self):
-        return (
-            f"create account {self.name} "
-            f"admin_name={self.admin_name}, admin_password='{self.admin_password}', "
-            f"email='{self.email}', edition={self.edition}, "
-            f"region='{self.region}' comment='{self.comment}';"
+        return Sql(
+            statement=(
+                f"create account {self.name} "
+                f"admin_name={self.admin_name}, admin_password='{self.admin_password}', "
+                f"email='{self.email}', edition={self.edition}, "
+                f"region='{self.region}' comment='{self.comment}';"
+            )
         )
 
     @property
     def drop(self):
-        return f"drop account {self.name};"
+        return Sql(statement=f"drop account {self.name};")
 
     @property
     def enable_replication(self):
-        return f"select system$global_account_set_parameter('{self.name}', 'ENABLE_ACCOUNT_DATABASE_REPLICATION', 'true');"
+        return Sql(
+            statement=f"select system$global_account_set_parameter('{self.name}', 'ENABLE_ACCOUNT_DATABASE_REPLICATION', 'true');"
+        )
 
     @property
     def disable_replication(self):
-        return f"select system$global_account_set_parameter('{self.name}', 'ENABLE_ACCOUNT_DATABASE_REPLICATION', 'false');"
+        return Sql(
+            statement=f"select system$global_account_set_parameter('{self.name}', 'ENABLE_ACCOUNT_DATABASE_REPLICATION', 'false');"
+        )
 
 
 @dataclass
@@ -90,12 +151,12 @@ class ManagedAccount(BaseSnowflakeObject):
 
 @dataclass
 class AwsApiIntegration(AccountObject):
-    api_aws_role_arn: str = ""  # Figure out a cleaner way to do this
+    api_aws_role_arn: str = ""
     api_provider: str = "aws_api_gateway"
     api_allowed_prefixes: List[str] = field(default_factory=list)
     api_blocked_prefixes: List[str] = field(default_factory=list)
     comment: str = ""
-    enabled: bool = True  # enabled by default
+    enabled: bool = True
 
     @property
     def resource_type(self):
@@ -118,7 +179,13 @@ class NetworkPolicy(AccountObject):
 @dataclass
 class ResourceMonitorTrigger:
     on_percent: int
-    do: str  # SUSPEND | SUSPEND_IMMEDIATE | NOTIFY
+    do: str
+
+    def _validate(self):
+        if self.do.lower() not in RESOURCE_MONITOR_TRIGGER_ACTIONS:
+            raise InvalidParamsException(
+                f"{self.do} is not a supported resource monitor trigger action. Options include: {RESOURCE_MONITOR_TRIGGER_ACTIONS}"
+            )
 
     @property
     def sql(self):
@@ -129,16 +196,31 @@ class ResourceMonitorTrigger:
 class ResourceMonitor(AccountObject):
     triggers: List[ResourceMonitorTrigger]
     credit_quota: int = 1
-    frequency: str = "never"  # must be MONTHLY | DAILY | WEEKLY | YEARLY | NEVER
-    start_timestamp: str = "immediately"  # Use strings because.... timestamp | "IMMEDIATELY"
-    end_timestamp: str = None  # String whyyy
+    frequency: str = "never"
+    start_timestamp: Union[datetime, str] = "immediately"
+    end_timestamp: datetime = None
+
+    def _validate(self):
+        if self.frequency.lower() not in RESOURCE_MONITOR_FREQUENCIES:
+            raise InvalidParamsException(
+                f"{self.frequency} is not a valid resource monitor frequency. Options include: {RESOURCE_MONITOR_FREQUENCIES}"
+            )
+        if (
+            isinstance(self.start_timestamp, str)
+            and self.start_timestamp.lower != "immediately"
+        ):
+            raise InvalidParamsException(
+                f"{self.start_timestamp} is not a valid resource monitor start. It must be 'immediately' or a corresponding datetime."
+            )
 
     @property
     def _settings_str(self):
+        end_timestamp_str = f"end_timestamp =  " if self.end_timestamp else ""
         return (
             f"with credit_quota={self.credit_quota} "
             f"frequency={self.frequency} "
             f"start_timestamp={self.start_timestamp} "
+            f"{end_timestamp_str}"
             f"triggers {self._format_triggers()}"
         )
 
@@ -165,10 +247,7 @@ class Share(AccountObject):
 
 @dataclass
 class Warehouse(AccountObject):
-    warehouse_size: str = "xsmall"  # enum this
-    min_cluster_count: int = 1
-    max_cluster_count: int = 1
-    scaling_policy: str = None  # enum this
+    warehouse_size: str = "xsmall"
     auto_suspend: int = 60
     auto_resume: bool = True
     initially_suspended: bool = True
@@ -176,24 +255,20 @@ class Warehouse(AccountObject):
     statement_queued_timeout_in_seconds: int = 600
     statement_timeout_in_seconds: int = 6000
     resource_monitor: ResourceMonitor = None
+    min_cluster_count: int = 1
+    max_cluster_count: int = 1
+    scaling_policy: str = None
     comment: str = ""
 
-    def __validate(self):
-        # Validate warehouse size param
-        if self.warehouse_size.lower() not in [
-            "xsmall",
-            "small",
-            "medium",
-            "large",
-            "xlarge",
-            "xxlarge",
-            "xxxlarge",
-        ]:
+    def _validate(self):
+        if self.warehouse_size.lower() not in WAREHOUSE_SIZES:
             raise InvalidParamsException(
-                f"{self.warehouse_size} is not a valid snowflake warehouse size"
+                f"{self.warehouse_size} is not a valid snowflake warehouse size. Options include: {WAREHOUSE_SIZES}"
+            )
+        if self.scaling_policy and self.scaling_policy.lower() not in SCALING_POLICIES:
+            raise InvalidParamsException(
+                f"{self.scaling_policy} is not a valid snowflake scaling policy. Options include: {SCALING_POLICIES}"
             )
-        # Validate scaling policy param
-        # Validate cluster count (if not enterprise account) param
 
 
 @dataclass
@@ -241,7 +316,7 @@ class User(BaseSnowflakeObject):
     timestamp_output_format: str = None
     timestamp_type_mapping: str = None
     timestamp_tz_output_format: str = None
-    timezone: str = None  # restrict to list of valid timezones
+    timezone: str = None
     time_input_format: str = None
     time_output_format: str = None
     transaction_default_isolation_level: str = None
@@ -278,10 +353,10 @@ class SecurityIntegrationOAuth(AccountObject):
     external_oauth_audience_list: str = None  # optional
     type: str = "external_oauth"
 
-    def __post_init__(self):
-        if self.external_oauth_type not in ["okta", "azure", "ping_federate", "custom"]:
+    def _validate(self):
+        if self.external_oauth_type.lower() not in EXTERNAL_OAUTH_TYPES:
             raise InvalidParamsException(
-                f"{self.external_oauth_type} is not a valid oauth type."
+                f"{self.external_oauth_type} is not a valid external oauth type. Options include: {EXTERNAL_OAUTH_TYPES}"
             )
 
     @property
