@@ -24,10 +24,9 @@ class Database(BaseSnowflakeObject):
     default_ddl_collation: str = None
 
     @property
-    def resource_type(self):
+    def resource_type_modifier(self):
         if self.transient:
-            return "transient database"
-        return "database"
+            return "transient"
 
 
 @dataclass
@@ -87,21 +86,23 @@ class DatabaseReplica(BaseSnowflakeObject):
         )
 
 
-# See: https://docs.snowflake.com/en/sql-reference/sql/create.html
 @dataclass
-class Schema(BaseSnowflakeObject):
+class BaseDatabaseObject(BaseSnowflakeObject):
     database: Database
+    comment: str = ""
+
+
+@dataclass
+class Schema(BaseDatabaseObject):
     transient: bool = False
     with_managed_access: bool = False
     data_retention_time_in_days: int = 1
     default_ddl_collation: str = None
-    comment: str = ""
 
     @property
-    def resource_type(self):
+    def resource_type_modifier(self):
         if self.transient:
-            return "transient schema"
-        return "schema"
+            return "transient"
 
     @property
     def fqn(self):
